--[[
-- Copyright (c) 2012  Anton Savoshchanka (aka AntonioModer)
--
-- This software is provided 'as-is', without any express or implied
-- warranty.  In no event will the authors be held liable for any damages
-- arising from the use of this software.
--
-- Permission is granted to anyone to use this software for any purpose,
-- including commercial applications, and to alter it and redistribute it
-- freely, subject to the following restrictions:
--
-- 1. The origin of this software must not be misrepresented; you must not
--    claim that you wrote the original software. If you use this software
--    in a product, an acknowledgment in the product documentation would be
--    appreciated but is not required.
-- 2. Altered source versions must be plainly marked as such, and must not be
--    misrepresented as being the original software.
-- 3. This notice may not be removed or altered from any source distribution.
--]]

--[[
[BUG1]  RAY. [FIX1]     RAY-.
--]]


require 'math2'
require 'newObj'
require 'sliceEngine'
require 'loveLoad'
require 'loveUpdate'
require 'loveDraw'

function worldRayCastCallback1(fixture, x, y, xn, yn, fraction)
    local hit = {};
    hit.fixture = fixture;
    hit.x, hit.y = x, y;
    hit.xn, hit.yn = xn, yn;
    hit.fraction = fraction;

    table.insert(ray1.hitList, hit);

    return 1 -- Continues with ray cast through all shapes.
end

function worldRayCastCallback2(fixture, x, y, xn, yn, fraction)
    local hit = {};
    hit.fixture = fixture;
    hit.x, hit.y = x, y;
    hit.xn, hit.yn = xn, yn;
    hit.fraction = fraction;

    table.insert(ray2.hitList, hit);

    return 1 -- Continues with ray cast through all shapes.
end

-- a is the first  fixture object in the collision. 
-- b is the second fixture object in the collision. 
-- coll is the contact object created.
function beginContact(a, b, coll)
	-- print('beginContact()')

	-- contact --
	local objA, objB = a:getUserData(), b:getUserData();
	
	objA.tabObjContact.normalX, objA.tabObjContact.normalY = coll:getNormal();
	objA.tabObjContact.x, objA.tabObjContact.y = coll:getPositions();

	objB.tabObjContact.normalX, objB.tabObjContact.normalY = coll:getNormal();
	objB.tabObjContact.x, objB.tabObjContact.y = coll:getPositions();	
	
	objCollA = objA; 																				-- set global var
	objCollB = objB;
	
end

function endContact(a, b, coll)
    
end

function preSolve(a, b, coll)

end

function postSolve(a, b, coll)

end

function love.load()
	print("\n".."start love.load() ")
	
	loveLoad()	
	-- TEST --

	print("end love.load() ".."\n")
end -- love.load()

function love.keypressed(key)
    if key == 'escape' then
        love.event.push('quit')                                                                        -- Quit the game.
    end 
end

function love.mousepressed(x, y, button)
-- print("love.mousepressed")
if button == "l" then	
	-- drag body --
	mouseGetPhysBodyPressed = true;																	-- global var
	for i, self in pairs(tabClass.Entity) do														
		if self.physShapeType == 4 or self.physShapeType == 3 then
			mouseGetPhysBody = 0;																	-- global var
			for tr=1, #self.physShapePoly do													
				if self.physFixturePoly[tr]:testPoint(love.mouse.getX(), love.mouse.getY()) then
					mouseGetPhysBody = self.physBody;												-- global var
					jointMouse = love.physics.newMouseJoint( mouseGetPhysBody, love.mouse.getPosition());	-- global var
				end	
			end				
		end
	end
	-- set ray --
	if love.keyboard.isDown("lctrl") or love.keyboard.isDown("rctrl") then
		countMousePress = countMousePress+1;														-- global var
		if countMousePress == 1 then														
			ray1.x1 = x;											
			ray1.y1 = y;
			ray1.x2 = x+0.01;												
			ray1.y2 = y+0.01;
			
			ray2.x1 = ray1.x2;											
			ray2.y1 = ray1.y2;
			ray2.x2 = x;												
			ray2.y2 = y;
		end			
	end
end
	
end	-- love.mousepressed

function love.mousereleased(x, y, button)
	-- print("love.mousereleased")
	if button == "l" then
		-- slice --
		if countMousePress == 1 then
			countMousePress = 0;																	-- global var
			
			ray1.x2 = x;												
			ray1.y2 = y;
			ray2.x1 = x;												
			ray2.y1 = y;
			
			ray1.hitList = nil;
			ray2.hitList = nil;
			ray1.hitList = {};
			ray2.hitList = {};
			

			-- BUG1 FIX (always before rayCast()) --
			if math.floor(ray1.x1) == math.floor(ray1.x2) then
				ray1.x2 = ray1.x2+0.1;
				ray2.x1 = ray1.x2;
			end
			if math.floor(ray1.y1) == math.floor(ray1.y2) then
				ray1.y2 = ray1.y2+0.1;
				ray2.y1 = ray1.y2;
			end	
			if ray1.x2 ~= 0 then
				physWorld:rayCast(ray1.x1, ray1.y1, ray1.x2, ray1.y2, worldRayCastCallback1)
				physWorld:rayCast(ray2.x1, ray2.y1, ray2.x2, ray2.y2, worldRayCastCallback2)
			end
			
			ray1Start = {}
			ray1End = {}
			ray1Start.x = ray1.x1
			ray1Start.y = ray1.y1
			ray1End.x = ray1.x2
			ray1End.y = ray1.y2	
			sliceConvex(ray1Start, ray1End, ray1.hitList, ray2.hitList, nil)

		end			
		-- drag body --
		mouseGetPhysBodyPressed = false;															-- global var
		mouseGetPhysBody = 0;																		-- global var
		if jointMouse ~= 0 then
			jointMouse:destroy();																	-- global var
			jointMouse = 0;
		end
		
	end
	
end

function love.update(dt)    
	-- print("\n".."start love.update() ")
    loveUpdate(dt)
	-- print("\n".."end love.update() ")
end

function love.draw()
    loveDraw()
end


